@@ -9,9 +9,8 @@
 # This file is part of BayesPy.
 #
 # BayesPy is free software: you can redistribute it and/or modify it
-# under the terms of the GNU General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# under the terms of the GNU General Public License version 3 as
+# published by the Free Software Foundation.
 #
 # BayesPy is distributed in the hope that it will be useful, but
 # WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -33,15 +32,10 @@ import scipy.sparse as sp # prefer CSC format
 #import time
 #import profile
 import scipy.spatial.distance as dist
-import scikits.sparse.distance as spdist
+#import scikits.sparse.distance as spdist
 
-import Nodes.ExponentialFamily as ef
-import utils
-
-import imp
-imp.reload(spdist)
-imp.reload(ef)
-imp.reload(utils)
+from . import node as ef
+from bayespy.utils import misc as utils
 
 # Covariance matrices can be either arrays or matrices so be careful
 # with products and powers! Use explicit multiply or dot instead of
