@@ -215,12 +215,15 @@ class VB():
             
         return L
 
-    def plot_iteration_by_nodes(self):
+    def plot_iteration_by_nodes(self, axes=None):
         """
         Plot the cost function per node during the iteration.
 
         Handy tool for debugging.
         """
+
+        if axes is None:
+            axes = plt.gca()
         
         D = len(self.l)
         N = self.iter
@@ -229,10 +232,10 @@ class VB():
         for (d, node) in enumerate(self.l):
             L[:,d] = self.l[node]
             legends += [node.name]
-        plt.plot(np.arange(N)+1, L)
-        plt.legend(legends, loc='lower right')
-        plt.title('Lower bound contributions by nodes')
-        plt.xlabel('Iteration')
+        axes.plot(np.arange(N)+1, L)
+        axes.legend(legends, loc='lower right')
+        axes.set_title('Lower bound contributions by nodes')
+        axes.set_xlabel('Iteration')
 
     def get_iteration_by_nodes(self):
         return self.l
@@ -341,27 +344,22 @@ class VB():
         """
         Plot the distribution of the given nodes (or all nodes)
         """
+
         if len(nodes) == 0:
             nodes = self.model
 
-        if not plt.isinteractive():
-            plt.ion()
-            redisable = True
-        else:
-            redisable = False
-
         for node in nodes:
             node = self[node]
             if node.has_plotter():
-                try:
-                    plt.figure(self._figures[node])
-                except:
-                    f = plt.figure()
-                    self._figures[node] = f.number
-                plt.clf()
-                node.plot()
-                plt.draw()
-                plt.show()
 
-        if redisable:
-            plt.ioff()
+                try:
+                    fignum = self._figures[node]
+                except KeyError:
+                    fig = plt.figure()
+                    self._figures[node] = fig.number
+                else:
+                    fig = plt.figure(num=fignum)
+
+                fig.clf()
+                node.plot(fig=fig)
+                fig.canvas.draw()
