@@ -1,4 +1,4 @@
-from . import utils
+from . import misc
 from . import linalg
 from . import random
 from . import optimize
